# Generated by roxygen2: do not edit by hand

S3method(VaR,bekkFit)
S3method(VaR,bekkForecast)
S3method(backtest,bekkFit)
S3method(bekk_fit,bekk)
S3method(bekk_fit,bekka)
S3method(bekk_fit,dbekk)
S3method(bekk_fit,dbekka)
S3method(bekk_fit,sbekk)
S3method(bekk_fit,sbekka)
S3method(logLik,bekkFit)
S3method(plot,backtest)
S3method(plot,bekkFit)
S3method(plot,bekkMC)
S3method(plot,var)
S3method(plot,virf)
S3method(portmanteau.test,bekkFit)
S3method(predict,bekk)
S3method(predict,bekka)
S3method(predict,dbekk)
S3method(predict,dbekka)
S3method(predict,sbekk)
S3method(predict,sbekka)
S3method(print,backtest)
S3method(print,bekkFit)
S3method(print,var)
S3method(process_object,bekkFit)
S3method(process_object,bekkSpec)
S3method(residuals,bekkFit)
S3method(simulate,bekk)
S3method(simulate,bekka)
S3method(simulate,dbekk)
S3method(simulate,dbekka)
S3method(simulate,sbekk)
S3method(simulate,sbekka)
S3method(summary,backtest)
S3method(summary,bekk)
S3method(summary,bekka)
S3method(summary,dbekk)
S3method(summary,dbekka)
S3method(summary,sbekk)
S3method(summary,sbekka)
S3method(virf,bekk)
S3method(virf,dbekk)
S3method(virf,sbekk)
export(VaR)
export(backtest)
export(bekk_fit)
export(bekk_spec)
export(portmanteau.test)
export(virf)
import(future)
import(future.apply)
import(ggfortify)
import(ggplot2)
import(grid)
import(gridExtra)
import(ks)
import(lubridate)
import(mathjaxr)
import(moments)
import(numDeriv)
import(pbapply)
import(reshape2)
import(stats)
import(utils)
import(xts)
importFrom(Rcpp,sourceCpp)
useDynLib(BEKKs)
