# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

diag_selection_mat <- function(n) {
    .Call('_BEKKs_diag_selection_mat', PACKAGE = 'BEKKs', n)
}

cut_mat_symmetric <- function(n) {
    .Call('_BEKKs_cut_mat_symmetric', PACKAGE = 'BEKKs', n)
}

cut_mat_asymmetric <- function(n) {
    .Call('_BEKKs_cut_mat_asymmetric', PACKAGE = 'BEKKs', n)
}

set_seed <- function(seed) {
    invisible(.Call('_BEKKs_set_seed', PACKAGE = 'BEKKs', seed))
}

valid_bekk <- function(C, A, G) {
    .Call('_BEKKs_valid_bekk', PACKAGE = 'BEKKs', C, A, G)
}

valid_asymm_bekk <- function(C, A, B, G, r, signs) {
    .Call('_BEKKs_valid_asymm_bekk', PACKAGE = 'BEKKs', C, A, B, G, r, signs)
}

valid_asymm_bekk_sim <- function(C, A, B, G, exp_indicator_value, signs) {
    .Call('_BEKKs_valid_asymm_bekk_sim', PACKAGE = 'BEKKs', C, A, B, G, exp_indicator_value, signs)
}

loglike_bekk <- function(theta, r) {
    .Call('_BEKKs_loglike_bekk', PACKAGE = 'BEKKs', theta, r)
}

loglike_asymm_bekk <- function(theta, r, signs) {
    .Call('_BEKKs_loglike_asymm_bekk', PACKAGE = 'BEKKs', theta, r, signs)
}

loglike_dbekk <- function(theta, r) {
    .Call('_BEKKs_loglike_dbekk', PACKAGE = 'BEKKs', theta, r)
}

loglike_asymm_dbekk <- function(theta, r, signs) {
    .Call('_BEKKs_loglike_asymm_dbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

score_bekk <- function(theta, r) {
    .Call('_BEKKs_score_bekk', PACKAGE = 'BEKKs', theta, r)
}

score_dbekk <- function(theta, r) {
    .Call('_BEKKs_score_dbekk', PACKAGE = 'BEKKs', theta, r)
}

score_asymm_bekk <- function(theta, r, signs) {
    .Call('_BEKKs_score_asymm_bekk', PACKAGE = 'BEKKs', theta, r, signs)
}

score_asymm_dbekk <- function(theta, r, signs) {
    .Call('_BEKKs_score_asymm_dbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

bhh_bekk <- function(r, theta, max_iter, crit) {
    .Call('_BEKKs_bhh_bekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit)
}

bhh_asymm_bekk <- function(r, theta, max_iter, crit, signs) {
    .Call('_BEKKs_bhh_asymm_bekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit, signs)
}

bhh_dbekk <- function(r, theta, max_iter, crit) {
    .Call('_BEKKs_bhh_dbekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit)
}

bhh_asymm_dbekk <- function(r, theta, max_iter, crit, signs) {
    .Call('_BEKKs_bhh_asymm_dbekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit, signs)
}

random_grid_search_BEKK <- function(r) {
    .Call('_BEKKs_random_grid_search_BEKK', PACKAGE = 'BEKKs', r)
}

random_grid_search_asymmetric_BEKK <- function(r, signs) {
    .Call('_BEKKs_random_grid_search_asymmetric_BEKK', PACKAGE = 'BEKKs', r, signs)
}

random_grid_search_dBEKK <- function(r) {
    .Call('_BEKKs_random_grid_search_dBEKK', PACKAGE = 'BEKKs', r)
}

random_grid_search_asymmetric_dBEKK <- function(r, signs) {
    .Call('_BEKKs_random_grid_search_asymmetric_dBEKK', PACKAGE = 'BEKKs', r, signs)
}

sigma_bekk <- function(r, C, A, G) {
    .Call('_BEKKs_sigma_bekk', PACKAGE = 'BEKKs', r, C, A, G)
}

sigma_bekk_asymm <- function(r, C, A, B, G, signs) {
    .Call('_BEKKs_sigma_bekk_asymm', PACKAGE = 'BEKKs', r, C, A, B, G, signs)
}

hesse_bekk <- function(theta, r) {
    .Call('_BEKKs_hesse_bekk', PACKAGE = 'BEKKs', theta, r)
}

hesse_dbekk <- function(theta, r) {
    .Call('_BEKKs_hesse_dbekk', PACKAGE = 'BEKKs', theta, r)
}

hesse_asymm_bekk <- function(theta, r, signs) {
    .Call('_BEKKs_hesse_asymm_bekk', PACKAGE = 'BEKKs', theta, r, signs)
}

hesse_asymm_dbekk <- function(theta, r, signs) {
    .Call('_BEKKs_hesse_asymm_dbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

eigen_value_decomposition <- function(A) {
    .Call('_BEKKs_eigen_value_decomposition', PACKAGE = 'BEKKs', A)
}

virf_bekk <- function(H_t, theta, shocks, periods) {
    .Call('_BEKKs_virf_bekk', PACKAGE = 'BEKKs', H_t, theta, shocks, periods)
}

virf_dbekk <- function(H_t, theta, shocks, periods) {
    .Call('_BEKKs_virf_dbekk', PACKAGE = 'BEKKs', H_t, theta, shocks, periods)
}

virf_bekka <- function(H_t, C, A, B, G, signs, expected_signs, shocks, periods) {
    .Call('_BEKKs_virf_bekka', PACKAGE = 'BEKKs', H_t, C, A, B, G, signs, expected_signs, shocks, periods)
}

virf_sbekk <- function(H_t, theta, shocks, periods) {
    .Call('_BEKKs_virf_sbekk', PACKAGE = 'BEKKs', H_t, theta, shocks, periods)
}

simulate_bekk_c <- function(theta, NoObs, n) {
    .Call('_BEKKs_simulate_bekk_c', PACKAGE = 'BEKKs', theta, NoObs, n)
}

simulate_bekka_c <- function(theta, NoObs, n, signs, expected_signs) {
    .Call('_BEKKs_simulate_bekka_c', PACKAGE = 'BEKKs', theta, NoObs, n, signs, expected_signs)
}

simulate_dbekk_c <- function(theta, NoObs, N) {
    .Call('_BEKKs_simulate_dbekk_c', PACKAGE = 'BEKKs', theta, NoObs, N)
}

simulate_dbekka_c <- function(theta, NoObs, N, signs, expected_signs) {
    .Call('_BEKKs_simulate_dbekka_c', PACKAGE = 'BEKKs', theta, NoObs, N, signs, expected_signs)
}

simulate_sbekk_c <- function(theta, NoObs, N) {
    .Call('_BEKKs_simulate_sbekk_c', PACKAGE = 'BEKKs', theta, NoObs, N)
}

simulate_sbekka_c <- function(theta, NoObs, N, signs, expected_signs) {
    .Call('_BEKKs_simulate_sbekka_c', PACKAGE = 'BEKKs', theta, NoObs, N, signs, expected_signs)
}

indicatorFunction <- function(r, signs) {
    .Call('_BEKKs_indicatorFunction', PACKAGE = 'BEKKs', r, signs)
}

expected_indicator_value <- function(r, signs) {
    .Call('_BEKKs_expected_indicator_value', PACKAGE = 'BEKKs', r, signs)
}

elimination_mat <- function(n) {
    .Call('_BEKKs_elimination_mat', PACKAGE = 'BEKKs', n)
}

commutation_mat <- function(n) {
    .Call('_BEKKs_commutation_mat', PACKAGE = 'BEKKs', n)
}

duplication_mat <- function(n) {
    .Call('_BEKKs_duplication_mat', PACKAGE = 'BEKKs', n)
}

inv_gen <- function(m) {
    .Call('_BEKKs_inv_gen', PACKAGE = 'BEKKs', m)
}

valid_sbekk <- function(C, a, g) {
    .Call('_BEKKs_valid_sbekk', PACKAGE = 'BEKKs', C, a, g)
}

valid_asymm_sbekk <- function(C, a, b, g, r, signs) {
    .Call('_BEKKs_valid_asymm_sbekk', PACKAGE = 'BEKKs', C, a, b, g, r, signs)
}

loglike_sbekk <- function(theta, r) {
    .Call('_BEKKs_loglike_sbekk', PACKAGE = 'BEKKs', theta, r)
}

loglike_asymm_sbekk <- function(theta, r, signs) {
    .Call('_BEKKs_loglike_asymm_sbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

score_sbekk <- function(theta, r) {
    .Call('_BEKKs_score_sbekk', PACKAGE = 'BEKKs', theta, r)
}

score_asymm_sbekk <- function(theta, r, signs) {
    .Call('_BEKKs_score_asymm_sbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

bhh_sbekk <- function(r, theta, max_iter, crit) {
    .Call('_BEKKs_bhh_sbekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit)
}

bhh_asymm_sbekk <- function(r, theta, max_iter, crit, signs) {
    .Call('_BEKKs_bhh_asymm_sbekk', PACKAGE = 'BEKKs', r, theta, max_iter, crit, signs)
}

hesse_sbekk <- function(theta, r) {
    .Call('_BEKKs_hesse_sbekk', PACKAGE = 'BEKKs', theta, r)
}

hesse_asymm_sbekk <- function(theta, r, signs) {
    .Call('_BEKKs_hesse_asymm_sbekk', PACKAGE = 'BEKKs', theta, r, signs)
}

sigma_sbekk <- function(r, C, a, g) {
    .Call('_BEKKs_sigma_sbekk', PACKAGE = 'BEKKs', r, C, a, g)
}

sigma_sbekk_asymm <- function(r, C, a, b, g, signs) {
    .Call('_BEKKs_sigma_sbekk_asymm', PACKAGE = 'BEKKs', r, C, a, b, g, signs)
}

random_grid_search_sBEKK <- function(r) {
    .Call('_BEKKs_random_grid_search_sBEKK', PACKAGE = 'BEKKs', r)
}

random_grid_search_asymmetric_sBEKK <- function(r, signs) {
    .Call('_BEKKs_random_grid_search_asymmetric_sBEKK', PACKAGE = 'BEKKs', r, signs)
}

YLagCr <- function(y, p) {
    .Call('_BEKKs_YLagCr', PACKAGE = 'BEKKs', y, p)
}

